"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const async_hooks_1 = __importDefault(require("async_hooks"));
function log(...args) {
    process.env.DEBUG_CLS && fs_1.default.writeSync(1, args.join(' ') + '\n');
}
const context = new Map();
async_hooks_1.default.createHook({
    init(asyncId, type, triggerAsyncId, resource) {
        log('Init: ', `${type}(asyncId=${asyncId}, parentAsyncId: ${triggerAsyncId})`);
        if (context.has(triggerAsyncId)) {
            context.set(asyncId, context.get(triggerAsyncId));
        }
    },
    before(asyncId) {
        log('Before: ', asyncId);
    },
    after(asyncId) {
        log('After: ', asyncId);
        context.delete(asyncId);
    },
    destroy(asyncId) {
        log('Destory: ', asyncId);
        context.delete(asyncId);
    }
}).enable();
class Session {
    constructor() {
        // cls by asyncId
        this.context = context;
    }
    get(key) {
        const asyncId = async_hooks_1.default.executionAsyncId();
        const store = this.context.get(asyncId);
        if (store) {
            return store.get(key);
        }
    }
    set(key, value) {
        const asyncId = async_hooks_1.default.executionAsyncId();
        const store = this.context.get(asyncId);
        if (store) {
            store.set(key, value);
        }
    }
    async scope(fn) {
        await Promise.resolve();
        const asyncId = async_hooks_1.default.executionAsyncId();
        log('\nScope:', asyncId);
        this.context.set(asyncId, new Map());
        const value = await fn();
        log('\nFinish: ', asyncId);
        return value;
    }
}
exports.default = Session;
